#ifndef _kernel_h_
#define _kernel_h_
#include <Python.h>


#ifdef __cplusplus
extern "C" {
#endif

/*
Another generally useful object type is an tuple of object pointers.
This is a mutable type: the tuple items can be changed (but not their
number).  Out-of-range indices or non-tuple objects are ignored.

*** WARNING *** PyTuple_SetItem does not increment the new item's reference
count, but does decrement the reference count of the item it replaces,
if not nil.  It does *decrement* the reference count if it is *not*
inserted in the tuple.  Similarly, PyTuple_GetItem does not increment the
returned item's reference count.
*/

staticforward PyTypeObject SaElement_Type;

typedef struct {
    PyObject_VAR_HEAD
    PyObject *x_attr;
    PyObject *x_content;
} SaElement;

extern int kernel_verbosity;
extern DL_IMPORT(PyTypeObject) PyTuple_Type;

extern void Satine_WriteInfo(const char *format, ...) __attribute__((format(printf, 1, 2)));;
extern void Satine_WriteMoreInfo(const char *format, ...) __attribute__((format(printf, 1, 2)));;

#ifdef _DEBUG
#define Satine_DebugInfo Satine_WriteMoreInfo
#else 
#define Satine_DebugInfo //
#endif

#define SaElement_Check(op) PyObject_TypeCheck(op, &SaElement_Type)
#define SaElement_CheckExact(op) ((op)->ob_type == &SaElement_Type)

extern PyObject* kernel_spaces;
extern PyObject* kernel_uri2mods;
extern PyObject* kernel_mod2uri;
extern PyObject* default_prefixes;
extern PyObject* default_uris;
extern int py2xml_strict;

extern PyObject*
kernel_xml2seq_c(char* s_buffer, int s_size, PyObject* suggested_uri, int options);
extern PyObject*
kernel_strip_c(PyObject *seq);

#ifdef __cplusplus
}
#endif
#endif 
