from satine import stream
from types import InstanceType, ListType, TupleType, DictType, StringType, UnicodeType, IntType, LongType
import odbc

import sys
import satine

class repository:
	def __init__(self, db, **keys):
		self.cursor = db.cursor()
		self.keys = keys
		for key in self.keys:
			self.keys[key] = self.keys[key].strip()
		try:
			self.cursor.execute("SELECT MAX(id) FROM documents")
			self.last_id = self.cursor.fetchone()[0]
			if self.last_id == None:
				self.last_id = 0
		except:
			self.create()
			self.last_id = 0
		
	def create(self):
		try:
			self.cursor.execute("DROP TABLE documents")
		except: pass
		try:
			self.cursor.execute("DROP TABLE keys")
		except: pass
		self.cursor.execute("CREATE TABLE documents (id INTEGER PRIMARY KEY, fragment INTEGER, document VARCHAR(255))")
		self.cursor.execute("CREATE TABLE keys (key_index VARCHAR(64), key_val VARCHAR(64), id INTEGER)")

	def __insert_doc(self, doc, id):
		for fragment in xrange(0, len(doc)/255+1):
			self.cursor.execute("INSERT INTO documents(id,fragment,document) VALUES(%i,%i,'%s')"
								% (id, fragment, doc[fragment*255:(fragment+1)*255].replace("'","''")) )

	def __insert_index(self, index_key, index_val, id):
		index_val = index_val.replace("'","''")
		self.cursor.execute("INSERT INTO keys(key_index, key_val, id) VALUES('%s','%s',%i)"
							% (index_key,index_val, id) )
	
	def put(self, xlist, prefixes=None):
		if not prefixes:
			prefixes = {}
		document = satine.py2xml(xlist, prefixes)
		self.last_id += 1
		self.__insert_doc(document, self.last_id)
		
		for key in self.keys:
			index_key = self.keys[key]
			values = xlist.query(index_key)
			
			for value in values:
				self.__insert_index(index_key, value, self.last_id)
	
	def __parse_cmd(self, cmd):
		pos = cmd.find("==")
		if pos != -1:
			key = cmd[0:pos].strip()
			val = cmd[pos+1:]
			return (key,val)
		pos = cmd.find("like ")
		if pos == -1:
			pos = cmd.find("=>")
		if pos == -1:
			pos = cmd.find(">")
		if pos == -1:
			pos = cmd.find("<=")
		if pos == -1:
			pos = cmd.find("<")
		key = cmd[0:pos].strip()
		val = cmd[pos]
		return (key,val)

	def __get_data(self, selection):
		sql = "SELECT document, id FROM documents WHERE 1=1"

		for index in selection:
			value = selection[index]
			sql += " AND id IN (SELECT id FROM keys WHERE keys.key_index='%s' AND keys.key_val %s" % (index, value)

		for i in range(len(selection)):
			sql += ")"
		sql += " ORDER BY id, fragment"
		
		self.cursor.execute(sql)
		return self.cursor.fetchall()
		
	def get(self, prefixes={}, **keys):
		for key in keys:
			keys[key] = "= '"+keys[key]+"'"
		return self.get_sql(prefixes, **keys)
	
	def get_sql(self, prefixes={}, **keys):
		selection = {}
		for key in keys:
			if key not in self.keys:
				raise "undefined key %s" % key
			selection[self.keys[key]] = keys[key]
		data = self.__get_data(selection)
		
		res = []
		xml = ""
		last_id = 0 
		for doc,id in data:
			if last_id != id:
				if xml:
					py = satine.xml2py(xml, prefixes)
					res.append(py)
					xml = ""
				last_id = id
			xml += doc
		if xml:
			py = satine.xml2py(xml, prefixes)
			res.append(py)
			
		return res
		
	def __get_id(self, index, expr):
		sql = "SELECT id FROM documents WHERE documents.id IN (SELECT id FROM keys "
		sql += "WHERE keys.key_index='%s' AND keys.key_val %s" % (self.index_key, expr)
		sql += ") ORDER BY id, fragment"
		
		self.cursor.execute(sql)
		return self.cursor.fetchall()

	def update(self, xlist, prefixes={},**keys):
		for key in keys:
			keys[key] = "= '"+keys[key]+"'"
		return self.update_sql(xlist, prefixes, **keys)

	def __remove_doc(self, id):
			sql ="DELETE FROM documents WHERE id = %s" %(id,)
			self.cursor.execute(sql)
		
	def __remove_index(self, index, id):
		self.cursor.execute("DELETE FROM keys WHERE keys.key_index='%s' AND keys.id=%s"
							% (index, id) )
	
	def update_sql(self, xlist,prefixes={}, **keys):
		selection = {}
		for key in keys:
			if key not in self.keys:
				raise "undefined key %s" % key
			selection[self.keys[key]] = keys[key]
		data = self.__get_data(selection)
		
		xml = satine.py2xml(xlist, prefixes)
		last_id = 0 
		for doc,id in data:
			self.__remove_doc(id)
			self.__insert_doc(xml, id)

			for key in self.keys:
				index_key = self.keys[key]
				values = xlist.query(index_key)
			
				self.__remove_index(index_key, id)
				for value in values:
					self.__insert_index(index_key, value, id)


def test():
	db = odbc.odbc("satine")
	py = satine.xml2py("""<message os="win" authour="Cristina">hello cri</message>""")
	rep = repository(db, os="<message>|os")
	rep.put(py)
	print rep.get(os="win")
	py.os="linux"
	rep.update(py, os="win")


if __name__ == "__main__":
	test()